/*----------------------------------------------------------------------------
 * Name:    Blinky.c
 * Purpose: LED Flasher
 * Note(s):
 *----------------------------------------------------------------------------*/

#include <stdio.h>
#include "cmsis_os2.h"                  // ::CMSIS:RTOS2

#include "RTE_Components.h"             // Component selection
#include  CMSIS_device_header           // Device header
#include "Board_LED.h"                  // ::Board Support:LED
#include "Board_Buttons.h"              // ::Board Support:Buttons

#if defined RTE_CMSIS_RTOS2_RTX5
/* stack size must be multiple of 8 Bytes */
#define APP_MAIN_STK_SZ (1024U)
extern uint64_t app_main_stk[APP_MAIN_STK_SZ / 8];
       uint64_t app_main_stk[APP_MAIN_STK_SZ / 8];
extern const osThreadAttr_t app_main_attr;
       const osThreadAttr_t app_main_attr = {
  .stack_mem  = &app_main_stk[0],
  .stack_size = sizeof(app_main_stk)
};
#endif

static volatile uint32_t delay_val = 500U;

static osThreadId_t tid_thrLED;                /* Thread id of thread: LED */
static osThreadId_t tid_thrBUT;                /* Thread id of thread: BUT */

/*----------------------------------------------------------------------------
  thrLED: blink LED
 *----------------------------------------------------------------------------*/
__NO_RETURN static void thrLED(void *argument) {
  uint32_t ledMax    = LED_GetCount();
  uint32_t ledNum    = 0;

  (void)argument;

  for (;;) {
    osThreadFlagsWait(0x0001, osFlagsWaitAny ,osWaitForever);
    LED_On(ledNum);                                           /* Turn specified LED on */

    osThreadFlagsWait(0x0001, osFlagsWaitAny ,osWaitForever);
    LED_Off(ledNum);                                          /* Turn specified LED off */

    ledNum++;                                                 /* Change LED number */
    if (ledNum >= ledMax) {
      ledNum = 0;                                             /* Restart with first LED */
    }
  }

}

/*----------------------------------------------------------------------------
  thrBUT: check button state
 *----------------------------------------------------------------------------*/
__NO_RETURN static void thrBUT(void *argument) {
  uint32_t butState = 0;;
  uint32_t butMax   = Buttons_GetCount();
  uint32_t butMask  = (1U << butMax) - 1;

  (void)argument;

  for (;;) {
    do {
      butState = Buttons_GetState();
    } while (butState & butMask);                             /* Wait while holding USER button */

    osThreadFlagsSet(tid_thrLED, 0x0001U);
    osDelay(delay_val);
  }

}

/*----------------------------------------------------------------------------
 * Application main thread
 *---------------------------------------------------------------------------*/
__NO_RETURN void app_main (void *argument);
__NO_RETURN void app_main (void *argument) {

  (void)argument;

  LED_Initialize ();
  Buttons_Initialize ();

  tid_thrLED = osThreadNew (thrLED, NULL, NULL);            /* create LED thread */
  if (tid_thrLED == NULL) { /* add error handling */ }

  tid_thrBUT = osThreadNew (thrBUT, NULL, NULL);            /* create BUT thread */
  if (tid_thrBUT == NULL) { /* add error handling */ }

  osThreadExit ();
//  for (;;) {}
}
